@@ -20,7 +20,7 @@
 /**
  * SECTION:gdata-parsable
  * @short_description: GData parsable object
- * @stability: Unstable
+ * @stability: Stable
  * @include: gdata/gdata-parsable.h
  *
  * #GDataParsable is an abstract class allowing easy implementation of an extensible parser. It is primarily extended by #GDataFeed and #GDataEntry,
@@ -209,17 +209,20 @@ _json_reader_dup_current_node (JsonReader *reader)
 		}
 	} else if (json_reader_is_object (reader) == TRUE) {
 		/* Object nodes require deep copies. */
-		gint i, members;
+		gint i;
+		gchar **members;
 		JsonObject *obj;
 
 		obj = json_object_new ();
 
-		for (i = 0, members = json_reader_count_members (reader); i < members; i++) {
-			json_reader_read_element (reader, i);
-			json_object_set_member (obj, json_reader_get_member_name (reader), _json_reader_dup_current_node (reader));
-			json_reader_end_element (reader);
+		for (i = 0, members = json_reader_list_members (reader); members[i] != NULL; i++) {
+			json_reader_read_member (reader, members[i]);
+			json_object_set_member (obj, members[i], _json_reader_dup_current_node (reader));
+			json_reader_end_member (reader);
 		}
 
+		g_strfreev (members);
+
 		value = json_node_new (JSON_NODE_OBJECT);
 		json_node_take_object (value, obj);
 	} else if (json_reader_is_array (reader) == TRUE) {
@@ -526,6 +529,7 @@ _gdata_parsable_new_from_json_node (GType parsable_type, JsonReader *reader, gpo
 		g_return_val_if_fail (json_reader_read_element (reader, i), NULL);
 
 		if (klass->parse_json (parsable, reader, user_data, error) == FALSE) {
+			json_reader_end_element (reader);
 			g_object_unref (parsable);
 			return NULL;
 		}
